@@ -1,8 +1,8 @@
 #! /bin/sh
 # Check for undefined symbols.
 
-for ENDIAN in -le -be; do
-for BITNESS in 32 64; do
+for ENDIAN in $TEST_ENDIAN; do
+for BITNESS in $TEST_BITS; do
 
 rm -rf tests/tmp/*
 
@@ -18,7 +18,7 @@ touch tests/tmp/empty
 [ "`depmod -F /empty 2>&1`" = "" ]
 
 # Without -F doesn't matter
-[ "`depmod -e 2>&1`" = "" ]
+[ "`depmod -e 2>&1`" = "WARNING: -e needs -E or -F" ]
 
 [ "`depmod -e -F /empty 2>&1`" = "WARNING: /lib/modules/$MODTEST_UNAME/unknown-$BITNESS.ko needs unknown symbol undefined" ]
 
